/*
    AnaCam.  Take anaglyph pictures on Android.
    Copyright (C) 2012  Daniel Kraft <d@domob.eu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package eu.domob.anacam;

import android.content.Context;
import android.content.SharedPreferences;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Handle the file output, including looking up the next number and
 * generating the file names from it, as well as writing the JPEG data
 * afterwards.
 */
public class OutputFiles
{

  /** File to use for left original.  */
  private File leftFile;
  /** File to use for right original.  */
  private File rightFile;
  /** File to use for combined anaglyph.  */
  private File anaglyphFile;

  /**
   * Create the object, not yet looking for the files.
   */
  public OutputFiles ()
  {
    leftFile = null;
    rightFile = null;
    anaglyphFile = null;
  }

  /**
   * Try to initialise the files.
   * @param pref Preferences object to get the path.
   * @param c Context to query string ressources.
   * @return Short name of found files.
   * @throws RuntimeException In case of failure.
   */
  public String initialise (SharedPreferences pref, Context c)
  {
    assert (leftFile == null);
    assert (rightFile == null);
    assert (anaglyphFile == null);

    /* Ensure the directory is there and usable.  */
    final String folder = pref.getString ("folder", null);
    final File dir = new File (folder);
    if (!dir.exists ())
      dir.mkdirs ();
    if (!dir.exists () || !dir.isDirectory ())
      {
        String msg = c.getString (R.string.error_noDir);
        msg = String.format (msg, folder);
        throw new RuntimeException (msg);
      }

    /* Look for the next still available number.  */
    int num = 0;
    while (true)
      {
        ++num;

        if (num >= 10000)
          throw new RuntimeException (c.getString (R.string.error_numsFull));

        String name;

        name = String.format (c.getString (R.string.file_left), num);
        leftFile = new File (dir, name);
        if (leftFile.exists ())
          continue;

        name = String.format (c.getString (R.string.file_right), num);
        rightFile = new File (dir, name);
        if (rightFile.exists ())
          continue;

        name = String.format (c.getString (R.string.file_anaglyph), num);
        anaglyphFile = new File (dir, name);
        if (anaglyphFile.exists ())
          continue;
        
        break;
      }

    /* Return short name.  */
    return String.format (c.getString (R.string.file_short), num);
  }

  /**
   * Write out the original files.
   * @param leftJpeg JPEG data for left file.
   * @param rightJpeg JPEG data for right file.
   * @throws IOException In case of IO error.
   */
  public void writeOriginals (byte[] leftJpeg, byte[] rightJpeg)
    throws IOException
  {
    FileOutputStream out;

    out = new FileOutputStream (leftFile);
    out.write (leftJpeg, 0, leftJpeg.length);
    out.close ();

    out = new FileOutputStream (rightFile);
    out.write (rightJpeg, 0, rightJpeg.length);
    out.close ();
  }

  /**
   * Return anaglyph output file.
   * @return Anaglyph output file.
   */
  public File getAnaglyphFile ()
  {
    return anaglyphFile;
  }

}
