/*
    AnaCam.  Take anaglyph pictures on Android.
    Copyright (C) 2012  Daniel Kraft <d@domob.eu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package eu.domob.anacam;

import android.app.Activity;

import android.content.SharedPreferences;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

import android.hardware.Camera;

import android.os.Debug;

import android.preference.PreferenceManager;

import android.util.Log;

import android.view.SurfaceHolder;
import android.view.SurfaceView;

import android.widget.ImageView;

import java.io.IOException;

/**
 * Handle the camera preview, including the dummy SurfaceView where the raw
 * preview is displayed and the ImageView to show the processed one.
 */
public class Preview implements SurfaceHolder.Callback, Camera.PreviewCallback
{

  /** The camera handle.  */
  private Camera cam;

  /** Store SharedPreferences object.  */
  private SharedPreferences pref;

  /** SurfaceView to use as dummy for the camera.  */
  private SurfaceView dummySurface;
  /** ImageView for real preview images.  */
  private ImageView previewImage;

  /** The anaglyph combiner.  */
  private Anaglyph ana;
  /** The YUV decoder.  */
  private YuvDecoder yuvDecoder;

  /** Remember whether we have a first picture already.  */
  private boolean haveFirst;

  /** Store camera parameters for performance.  */
  private Camera.Parameters param;

  /** Store the current preview bitmap here.  Recycle when replacing it.  */
  private Bitmap previewBmp;

  /**
   * Create it and register the callback to set up camera when we are ready.
   * @param c The context.
   * @param p Preferences object.
   */
  public Preview (Activity c, SharedPreferences p)
  {
    cam = null;
    param = null;
    pref = p;

    previewBmp = null;

    ana = new Anaglyph (pref);
    setFirstPicture (null);

    yuvDecoder = new YuvDecoder ();

    dummySurface = (SurfaceView) c.findViewById (R.id.preview_surface);
    previewImage = (ImageView) c.findViewById (R.id.real_preview);

    SurfaceHolder holder = dummySurface.getHolder ();
    holder.addCallback (this);
    holder.setType (SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
  }

  /**
   * Take a picture by the camera.
   * @param cb Callback to use when data is available.
   */
  public void takePicture (Camera.PictureCallback cb)
  {
    cam.takePicture (null, null, null, cb);
  }

  /**
   * Set first picture already taken.
   * @param f JPEG data of first picture.
   */
  public void setFirstPicture (byte[] f)
  {
    if (f != null)
      {
        BitmapFactory.Options opts;
        final Camera.Size size = param.getPreviewSize ();

        opts = new BitmapFactory.Options ();
        opts.inJustDecodeBounds = true;
        BitmapFactory.decodeByteArray (f, 0, f.length, opts);

        final float sampleX = opts.outWidth / (float) size.width;
        final float sampleY = opts.outHeight / (float) size.height;
        final int sample = (int) Math.floor (Math.min (sampleX, sampleY));
        Log.d (AnaCam.TAG, String.format ("Subsampling: %d", sample));
        
        opts = new BitmapFactory.Options ();
        opts.inSampleSize = sample;
        final Bitmap bmp = BitmapFactory.decodeByteArray (f, 0, f.length, opts);
        final Bitmap scaled = Bitmap.createScaledBitmap (bmp, size.width,
                                                         size.height, true);
        AnaCam.logMemory ("after scaling bitmaps");
        bmp.recycle ();

        final MyBitmap myScaled = new MyBitmap (scaled);
        if (wantGray ())
          myScaled.toGrayscale ();
        ana.setFirst (myScaled);
        scaled.recycle ();
      }
    else
      ana.setFirst (null);

    haveFirst = (f != null);
  }

  /**
   * Start preview.  This is used to re-start it after taking pictures.
   */
  public void start ()
  {
    if (AnaCam.PROFILING == AnaCam.PROFILE_SECOND)
      Debug.startMethodTracing ();
    cam.startPreview ();
  }

  /**
   * Query for the camera's jpeg quality.
   * @return Camera jpeg quality.
   */
  public int getJpegQuality ()
  {
    return param.getJpegQuality ();
  }

  /**
   * Free all unneeded buffers to make as much memory as possible available.
   */
  public void freeBuffers ()
  {
    ana.freeBuffers ();
    yuvDecoder.freeBuffers ();
  }

  /**
   * When the surface is created, set up the camera preview.
   * @param holder Surface holder.
   */
  public void surfaceCreated (SurfaceHolder holder)
  {
    assert (cam == null);
    cam = Camera.open ();
    cam.setPreviewCallback (this);
    try
      {
        cam.setPreviewDisplay (holder);
      }
    catch (IOException exc)
      {
        exc.printStackTrace ();
      }

    param = cam.getParameters ();
    Log.i (AnaCam.TAG, String.format ("Preview is %dx%d.",
           param.getPreviewSize ().width, param.getPreviewSize ().height));
    Log.i (AnaCam.TAG,
           String.format ("Format is %d.", param.getPreviewFormat ()));
    Log.i (AnaCam.TAG,
           String.format ("Quality is %d.", param.getJpegQuality ()));

    if (AnaCam.PROFILING == AnaCam.PROFILE_FIRST)
      Debug.startMethodTracing ();
    cam.startPreview ();
  }

  /**
   * Clean up everything when the surface is destroyed.
   * @param holder Surface holder.
   */
  public void surfaceDestroyed (SurfaceHolder holder)
  {
    cam.setPreviewCallback (null);
    cam.stopPreview ();

    if (AnaCam.PROFILING == AnaCam.PROFILE_FIRST
        || AnaCam.PROFILING == AnaCam.PROFILE_SECOND)
      Debug.stopMethodTracing ();

    cam.release ();
    cam = null;
    param = null;
  }

  /**
   * Surface changed, nothing to do.
   * @param holder Surface holder.
   * @param fmt Format.
   * @param w Width.
   * @param h Height.
   */
  public void surfaceChanged (SurfaceHolder holder, int fmt, int w, int h)
  {
    Log.i (AnaCam.TAG, String.format ("Display size is %dx%d.", w, h));
    // Do nothing more.
  }

  /**
   * Handle camera preview frame.
   * @param data Data array.
   * @param c The camera that gave the data.
   */
  public void onPreviewFrame (byte[] data, Camera c)
  {
    assert (c == cam);
    final int w = param.getPreviewSize ().width;
    final int h = param.getPreviewSize ().height;

    final int fmt = param.getPreviewFormat ();
    yuvDecoder.setInput (data, w, h, fmt);
    MyBitmap bmp = yuvDecoder.decode (wantGray ());
    data = null;

    MyBitmap preview;
    if (haveFirst)
      {
        ana.setSecond (bmp);
        bmp = null;
        preview = ana.combine (false);
        ana.setSecond (null);
      }
    else
      preview = bmp;

    final Bitmap newBmp = Bitmap.createBitmap (preview.getData (), w, h,
                                               Bitmap.Config.ARGB_8888);
    preview = null;
    previewImage.setImageBitmap (newBmp);
    if (previewBmp != null)
      previewBmp.recycle ();
    previewBmp = newBmp;
  }

  /**
   * Check the preferences to see whether we want grayscale preview.
   * @return True if yes.
   */
  private boolean wantGray ()
  {
    if (pref.getBoolean ("grayscale", false))
      return true;
    return pref.getBoolean ("grayPreview", false);
  }

}
