/*
    AnaCam.  Take anaglyph pictures on Android.
    Copyright (C) 2012  Daniel Kraft <d@domob.eu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package eu.domob.anacam;

import android.content.Context;
import android.content.SharedPreferences;

import android.os.Handler;
import android.os.Message;

import android.util.Log;

/**
 * The thread performing the saving action.
 */
public class SaverThread extends Thread
{

  /** Message code when saving succeeded.  */
  public static final int MSG_OK = 0;
  /** Message code in case of error.  */
  public static final int MSG_ERROR = 1;
  /** Message sent always when we should dismiss the action.  */
  public static final int MSG_DISMISS = 2;

  /** Context to use for string ressources.  */
  private Context context;
  /** SharedPreferences handler.  */
  private SharedPreferences pref;

  /** JPEG data of first picture.  */
  private byte[] firstJpeg;
  /** JPEG data of second picture.  */
  private byte[] secondJpeg;

  /** Quality for saving JPEG.  */
  private int quality;

  /** Handler for messages.  */
  private Handler handler;

  /**
   * Create it and give access to all the necessary data.
   * @param first First picture.
   * @param second Second picture.
   * @param q Quality for saving JPEG.
   * @param p Preferences object to use.
   * @param c Context to use.
   * @param h Handler object.
   */
  public SaverThread (byte[] first, byte[] second, int q,
                      SharedPreferences p, Context c, Handler h)
  {
    super ();
    firstJpeg = first;
    secondJpeg = second;
    quality = q;
    pref = p;
    context = c;
    handler = h;
  }

  /**
   * Perform the run action.
   */
  @Override
  public void run ()
  {
    try
      {
        final OutputFiles files = new OutputFiles ();
        final String shortFile = files.initialise (pref, context);

        final String order = pref.getString ("order", null);
        if (order.equals ("left"))
          files.writeOriginals (firstJpeg, secondJpeg);
        else
          files.writeOriginals (secondJpeg, firstJpeg);

        if (!pref.getBoolean ("onlyOriginals", false))
          {
            final boolean gray = pref.getBoolean ("grayscale", false);

            Anaglyph anaBuilder = new Anaglyph (pref);
            MyBitmap my;

            AnaCam.logMemory ("before allocating first bitmap");
            my = new MyBitmap (firstJpeg);
            firstJpeg = null;
            if (gray)
              my.toGrayscale ();
            anaBuilder.setFirst (my);

            AnaCam.logMemory ("before allocating second bitmap");
            my = new MyBitmap (secondJpeg);
            secondJpeg = null;
            if (gray)
              my.toGrayscale ();
            anaBuilder.setSecond (my);

            AnaCam.logMemory ("before combining anaglyph");
            my = anaBuilder.combine (true);
            anaBuilder = null;

            AnaCam.logMemory ("before saving");
            my.saveJpeg (files.getAnaglyphFile (), quality);
          }
        else
          {
            firstJpeg = null;
            secondJpeg = null;
          }

        String str = context.getString (R.string.saveSuccess);
        str = String.format (str, shortFile);
        handler.sendMessage (Message.obtain (handler, MSG_OK, str));
      }
    catch (Exception exc)
      {
        final Message msg = Message.obtain (handler, MSG_ERROR,
                                            exc.getMessage ());
        handler.sendMessage (msg);
      }
    finally
      {
        handler.sendEmptyMessage (MSG_DISMISS);
      }
  }

}
